@@ -1,14 +1,19 @@
 
-from io import TextIOWrapper
 import json
 import os
 import tempfile
 import time
+from typing import TYPE_CHECKING
+from liblo import Address
 
 import ray
 
 from daemon_tools import RS, dirname, Terminal
 from server_sender import ServerSender
+from jack_renaming_tools import group_belongs_to_client
+
+if TYPE_CHECKING:
+    from session_signaled import SignaledSession
 
 JSON_PATH = 'ray_canvas.json'
 
@@ -26,7 +31,7 @@ def _get_version_tuple_json_dict(json_contents: dict) -> tuple[int, int, int]:
 
 
 class CanvasSaver(ServerSender):
-    def __init__(self, session):
+    def __init__(self, session: 'SignaledSession'):
         ServerSender.__init__(self)
         self.session = session
 
@@ -47,7 +52,7 @@ class CanvasSaver(ServerSender):
             try:
                 json_contents = json.load(f)
             except json.JSONDecodeError:
-                Terminal.message("Failed to load patchcanvas config file %s" % f)
+                Terminal.message(f"Failed to load patchcanvas config file {f}")
 
             # Old group_position port_types_view norm was
             # full_view = AUDIO | MIDI (1|2 = 3)
@@ -73,15 +78,17 @@ class CanvasSaver(ServerSender):
                 gpos.write_from_dict(gpos_dict)
                 if needs_port_types_view_convert and gpos.port_types_view == 3:
                     gpos.port_types_view = 15
-                self.group_positions_config.append(gpos)
+                
+                if not [g for g in self.group_positions_config if g.is_same(gpos)]:
+                    self.group_positions_config.append(gpos)
 
             for pg_dict in pg_list:
                 portgroup = ray.PortGroupMemory()
                 portgroup.write_from_dict(pg_dict)
                 self.portgroups.append(portgroup)
 
-    def get_all_group_positions(self)->list:
-        group_positions_config_exclu = []
+    def get_all_group_positions(self) -> list[ray.GroupPosition]:
+        group_positions_config_exclu = list[ray.GroupPosition]()
 
         for gpos_cf in self.group_positions_config:
             for gpos_ss in self.group_positions_session:
@@ -126,7 +133,7 @@ class CanvasSaver(ServerSender):
                               '/ray/gui/patchbay/update_group_position',
                               *gpos.spread())
 
-    def send_all_group_positions(self, src_addr):
+    def send_all_group_positions(self, src_addr: Address):
         if ray.are_on_same_machine(self.get_server_url(), src_addr.url):
             # GUI is on the same machine than the daemon
             # send group positions via a tmp file because they can be many
@@ -192,7 +199,6 @@ class CanvasSaver(ServerSender):
         gp = ray.GroupPosition.new_from(*args)
         for group_positions in (self.group_positions_session,
                                 self.group_positions_config):
-            
             for gpos in group_positions:
                 if gpos.is_same(gp):
                     gpos.update(*args)
@@ -217,20 +223,26 @@ class CanvasSaver(ServerSender):
             except json.JSONDecodeError:
                 Terminal.message("Failed to load session canvas file %s" % f)
 
-            needs_port_types_view_convert = False
+            session_version = (0, 14, 0)
 
             if isinstance(json_contents, dict):
                 if 'group_positions' in json_contents.keys():
                     gpos_list : list[dict]() = json_contents['group_positions']
-                if _get_version_tuple_json_dict(json_contents) < (0, 13, 0):
-                    needs_port_types_view_convert = True
+                session_version = _get_version_tuple_json_dict(json_contents)
 
             for gpos_dict in gpos_list:
                 gpos = ray.GroupPosition()
                 gpos.write_from_dict(gpos_dict)
-                if needs_port_types_view_convert and gpos.port_types_view == 3:
-                    gpos.port_types_view = 15
-                self.group_positions_session.append(gpos)
+                if session_version < (0, 13, 0):
+                    if gpos.port_types_view == 3:
+                        gpos.port_types_view = 15
+                
+                elif session_version < (0, 14, 0):
+                    if gpos.port_types_view == 15:
+                        gpos.port_types_view = 31
+                
+                if not [g for g in self.group_positions_session if g.is_same(gpos)]:
+                    self.group_positions_session.append(gpos)
 
     def save_json_session_canvas(self, session_path: str):
         session_json_path = "%s/.%s" % (session_path, JSON_PATH)
@@ -274,3 +286,16 @@ class CanvasSaver(ServerSender):
             self.portgroups.remove(portgroup)
 
         self.portgroups.append(new_portgroup)
+
+    def client_jack_name_changed(self, old_jack_name: str, new_jack_name: str):
+        server = self.session.get_server()
+        
+        for gpos in self.group_positions_session:
+            if group_belongs_to_client(gpos.group_name, old_jack_name):
+                gpos.group_name = gpos.group_name.replace(
+                    old_jack_name, new_jack_name, 1)
+
+                if server is not None:
+                    server.send_gui(
+                        '/ray/gui/patchbay/update_group_position',
+                        *gpos.spread())
