@@ -1,28 +1,43 @@
 import argparse
 import os
 import sys
+from dataclasses import dataclass
+from typing import TYPE_CHECKING
+from pathlib import Path
+
 from PyQt5.QtCore import (QCoreApplication, QStandardPaths, QSettings,
                           QDateTime, QLocale)
+import liblo
+
 
 import ray
 
+if TYPE_CHECKING:
+    from client import Client
+
+
 settings = QSettings()
 
-def dirname(*args)->str:
+# remember here associated desktop files for executables
+# key: executable, value: desktop_file name
+exec_and_desktops = dict[str, str]()
+
+
+def dirname(*args) -> str:
     return os.path.dirname(*args)
 
-def basename(*args)->str:
+def basename(*args) -> str:
     return os.path.basename(*args)
 
-def get_app_config_path()->str:
+def get_app_config_path() -> str:
     return "%s/%s" % (
             QStandardPaths.writableLocation(QStandardPaths.ConfigLocation),
             QCoreApplication.organizationName())
 
-def get_code_root()->str:
-    return dirname(dirname(dirname(os.path.realpath(__file__))))
+def get_code_root() -> Path:
+    return Path(__file__).parent.parent.parent
 
-def is_pid_child_of(child_pid, parent_pid)->bool:
+def is_pid_child_of(child_pid, parent_pid) -> bool:
     if child_pid < parent_pid:
         return False
 
@@ -56,7 +71,7 @@ def is_pid_child_of(child_pid, parent_pid)->bool:
 
     return False
 
-def highlight_text(string)->str:
+def highlight_text(string) -> str:
     if "'" in string:
         return '"%s"' % string
     return "'%s'" % string
@@ -74,9 +89,9 @@ def init_daemon_tools():
     RS.set_favorites(ray.get_list_in_settings(l_settings, 'daemon/favorites'))
     TemplateRoots.init_config()
 
-def get_git_default_un_and_ignored(executable:str)->tuple:
-    ignored = []
-    unignored = []
+def get_git_default_un_and_ignored(executable:str) -> tuple[list[str], list[str]]:
+    ignored = list[str]()
+    unignored = list[str]()
 
     if executable in ('luppp', 'sooperlooper', 'sooperlooper_nsm'):
         unignored.append('.wav')
@@ -87,10 +102,18 @@ def get_git_default_un_and_ignored(executable:str)->tuple:
     return (ignored, unignored)
 
 
+@dataclass
+class AppTemplate:
+    template_name: str
+    template_client: 'Client'
+    display_name: str
+    templates_root: str
+
+
 class RS:
     settings = QSettings()
     non_active_clients = []
-    favorites = []
+    favorites = list[ray.Favorite]()
 
     @classmethod
     def set_settings(cls, settings):
@@ -103,15 +126,31 @@ class RS:
         cls.non_active_clients = nalist
 
     @classmethod
-    def set_favorites(cls, favorites):
-        cls.favorites = favorites
-
+    def set_favorites(cls, favorites: list[ray.Favorite]):
+        cls.favorites.clear()
+        
+        for fav in favorites:
+            fav_dict = fav.__dict__
+            if (isinstance(fav_dict.get('name'), str)
+                    and isinstance(fav_dict.get('factory'), bool)
+                    and isinstance(fav_dict.get('icon'), str)):
+                display_name = fav_dict.get('display_name')
+                if not isinstance(display_name, str):
+                    display_name = ''
+                
+                if not display_name:
+                    display_name = fav_dict.get('name')
+                cls.favorites.append(ray.Favorite(
+                    fav_dict.get('name'),
+                    fav_dict.get('icon'),
+                    fav_dict.get('factory'),
+                    display_name))
 
 class TemplateRoots:
     net_session_name = ".ray-net-session-templates"
-    factory_sessions = "%s/session_templates" % get_code_root()
-    factory_clients = "%s/client_templates"  % get_code_root()
-    factory_clients_xdg = "/etc/xdg/raysession/client_templates"
+    factory_sessions = get_code_root() / 'session_templates'
+    factory_clients = get_code_root() / 'client_templates'    
+    factory_clients_xdg = Path('/etc/xdg/raysession/client_templates')
 
     @classmethod
     def init_config(cls):
@@ -120,8 +159,8 @@ class TemplateRoots:
         else:
             app_config_path = get_app_config_path()
 
-        cls.user_sessions = "%s/session_templates" % app_config_path
-        cls.user_clients = "%s/client_templates"  % app_config_path
+        cls.user_sessions = Path(app_config_path) / 'session_templates'
+        cls.user_clients = Path(app_config_path) / 'client_templates'
 
 
 class Terminal:
@@ -197,16 +236,19 @@ class Terminal:
 
 class CommandLineArgs(argparse.Namespace):
     session_root = ''
+    hidden = False
     osc_port = 0
     findfreeport = True
-    gui_url = None
-    gui_port = 0
+    control_url: liblo.Address = None
+    gui_url: liblo.Address = None
+    gui_port: liblo.Address = 0
     gui_pid = 0
     config_dir = ''
     debug = False
     debug_only = False
     no_client_messages = False
     session = ''
+    no_options = False
 
     @classmethod
     def eat_attributes(cls, parsed_args):
