@@ -1,8 +1,14 @@
 import os
+from typing import TYPE_CHECKING, Union
 from PyQt5.QtXml import QDomDocument, QDomElement
+from pathlib import Path
 
 import ray
 
+if TYPE_CHECKING:
+    from session import Session
+    from osc_server_thread import OscServerThread
+
 instance = None
 
 class Daemon:
@@ -14,10 +20,11 @@ class Daemon:
     user = ""
     not_default = False
 
+
 class MultiDaemonFile:
     file_path = '/tmp/RaySession/multi-daemon.xml'
 
-    def __init__(self, session, server):
+    def __init__(self, session: 'Session', server: 'OscServerThread'):
         self.session = session
         self.server = server
 
@@ -77,7 +84,7 @@ class MultiDaemonFile:
         except:
             return
 
-    def _set_attributes(self, element):
+    def _set_attributes(self, element: QDomElement):
         element.setAttribute('net_daemon_id', self.server.net_daemon_id)
         element.setAttribute('root', self.session.root)
         element.setAttribute('session_path', self.session.path)
@@ -189,7 +196,9 @@ class MultiDaemonFile:
 
         return True
 
-    def is_free_for_session(self, session_path)->bool:
+    def is_free_for_session(self, session_path: Union[str, Path]) -> bool:
+        session_path = str(session_path)
+        
         if not self._open_file():
             return True
 
