@@ -1,25 +1,39 @@
+import logging
 import os
+import shlex
 import sys
 import random
 import shutil
 import subprocess
 import time
-import liblo
+from typing import TYPE_CHECKING
+from pathlib import Path
+import xml.etree.ElementTree as ET
 
+import liblo
 from PyQt5.QtCore import QCoreApplication
-from PyQt5.QtXml import QDomDocument
 
 import ray
 from signaler import Signaler
 from multi_daemon_file import MultiDaemonFile
-from daemon_tools import (TemplateRoots, CommandLineArgs, Terminal, RS,
-                          get_code_root)
+from daemon_tools import (
+    TemplateRoots,
+    CommandLineArgs,
+    Terminal,
+    RS,
+    get_code_root)
+from xml_tools import XmlElement
+from terminal_starter import which_terminal
+
+if TYPE_CHECKING:
+    from session_signaled import SignaledSession
 
 instance = None
 signaler = Signaler.instance()
 _translate = QCoreApplication.translate
+_logger = logging.getLogger(__name__)
 
-def _path_is_valid(path: str)->bool:
+def _path_is_valid(path: str) -> bool:
     if path.startswith(('./', '../')):
         return False
 
@@ -49,7 +63,7 @@ def ray_method(path, types):
     return decorated
 
 class Controller:
-    addr = None
+    addr: liblo.Address = None
     pid = 0
 
 
@@ -61,7 +75,7 @@ class GuiAdress(liblo.Address):
 # ClientCommunicating contains NSM protocol.
 # OSC paths have to be never changed.
 class ClientCommunicating(liblo.ServerThread):
-    def __init__(self, session, osc_num=0):
+    def __init__(self, session: 'SignaledSession', osc_num=0):
         liblo.ServerThread.__init__(self, osc_num)
         self.session = session
 
@@ -69,9 +83,9 @@ class ClientCommunicating(liblo.ServerThread):
         self._net_master_daemon_url = ''
         self._list_asker_addr = None
 
-        self.gui_list = []
-        self.controller_list = []
-        self.monitor_list = []
+        self.gui_list = list[GuiAdress]()
+        self.controller_list = list[Controller]()
+        self.monitor_list = list[liblo.Address]()
         self.server_status = ray.ServerStatus.OFF
         self.is_nsm_locked = False
         self.not_default = False
@@ -214,12 +228,18 @@ class ClientCommunicating(liblo.ServerThread):
             return False
 
     @ray_method('/nsm/server/broadcast', None)
-    def nsmServerBroadcast(self, path, args, types, src_addr):
+    def nsmServerBroadcast(self, path, args, types, src_addr: liblo.Address):
         if not args:
             return False
 
+        if not isinstance(args[0], str):
+            return False
+
         #don't allow clients to broadcast NSM commands
-        if args[0].startswith('/nsm/') or args[0].startswith('/ray'):
+        follow_path = args[0]
+        if not isinstance(follow_path, str):
+            return False
+        if follow_path.startswith(('/nsm/', '/ray/')):
             return False
 
         for client in self.session.clients:
@@ -329,6 +349,9 @@ class ClientCommunicating(liblo.ServerThread):
         self.send(src_addr, '/minor_error', path, ray.Err.UNKNOWN_MESSAGE,
                   "unknown osc message: %s %s" % (path, types))
 
+    def send_gui(self, *args):
+        # should be overclassed
+        pass
 
 class OscServerThread(ClientCommunicating):
     def __init__(self, session, osc_num=0):
@@ -346,9 +369,9 @@ class OscServerThread(ClientCommunicating):
             'daemon/options',
             ray.Option.BOOKMARK_SESSION
             + ray.Option.SNAPSHOTS
-            + ray.Option.SESSION_SCRIPTS
-            + ray.Option.GUI_STATES,
-            type=int)
+            + ray.Option.SESSION_SCRIPTS,
+            type=int
+        )
 
         if CommandLineArgs.no_options:
             self.options = 0
@@ -367,16 +390,25 @@ class OscServerThread(ClientCommunicating):
             'factory': [], 'user': []}
 
         self.session_to_preview = ''
+        
+        self._terminal_command_is_default = True
+        self.terminal_command = RS.settings.value(
+            'daemon/terminal_command', '', type=str)
+        if self.terminal_command:
+            self._terminal_command_is_default = False
+        else:
+            self.terminal_command = shlex.join(
+                which_terminal(title='RAY_TERMINAL_TITLE'))
 
         global instance
         instance = self
 
     @staticmethod
-    def getInstance():
+    def get_instance() -> 'OscServerThread':
         return instance
 
     @ray_method('/ray/server/gui_announce', 'sisii')
-    def rayGuiGui_announce(self, path, args, types, src_addr):
+    def rayGuiGui_announce(self, path, args, types, src_addr: liblo.Address):
         (version, int_nsm_locked, net_master_daemon_url,
          gui_pid, net_daemon_id) = args
 
@@ -402,7 +434,7 @@ class OscServerThread(ClientCommunicating):
         self.announce_gui(src_addr.url, nsm_locked, is_net_free, gui_pid)
 
     @ray_method('/ray/server/gui_disannounce', '')
-    def rayGuiGui_disannounce(self, path, args, types, src_addr):
+    def rayGuiGui_disannounce(self, path, args, types, src_addr: liblo.Address):
         for addr in self.gui_list:
             if ray.are_same_osc_port(addr.url, src_addr.url):
                 break
@@ -425,7 +457,6 @@ class OscServerThread(ClientCommunicating):
     @ray_method('/ray/server/ask_for_patchbay', '')
     def rayServerGetPatchbayPort(self, path, args, types, src_addr):
         patchbay_file = '/tmp/RaySession/patchbay_daemons/' + str(self.port)
-        patchbay_port = 0
 
         if not os.path.exists(patchbay_file):
             return True
@@ -480,7 +511,7 @@ class OscServerThread(ClientCommunicating):
         self.send(src_addr, '/reply', path, 'announced')
 
     @ray_method('/ray/server/controller_disannounce', '')
-    def rayServerControllerDisannounce(self, path, args, types, src_addr):
+    def rayServerControllerDisannounce(self, path, args, types, src_addr: liblo.Address):
         for controller in self.controller_list:
             if controller.addr.url == src_addr.url:
                 break
@@ -498,7 +529,7 @@ class OscServerThread(ClientCommunicating):
         self.send(src_addr, '/reply', path, 'announced')
     
     @ray_method('/ray/server/monitor_quit', '')
-    def rayServerMonitorDisannounce(self, path, args, types, src_addr):
+    def rayServerMonitorDisannounce(self, path, args, types, src_addr: liblo.Address):
         for monitor_addr in self.monitor_list:
             if monitor_addr.url == src_addr.url:
                 break
@@ -509,7 +540,7 @@ class OscServerThread(ClientCommunicating):
         self.send(src_addr, '/reply', path, 'monitor exit')
 
     @ray_method('/ray/server/set_nsm_locked', '')
-    def rayServerSetNsmLocked(self, path, args, types, src_addr):
+    def rayServerSetNsmLocked(self, path, args, types, src_addr: liblo.Address):
         self.is_nsm_locked = True
         self._nsm_locker_url = src_addr.url
 
@@ -535,7 +566,7 @@ class OscServerThread(ClientCommunicating):
 
     @ray_method('/ray/server/change_root', 's')
     def rayServerChangeRoot(self, path, args, types, src_addr):
-        new_root = args[0]
+        new_root: str = args[0]
         if not(new_root.startswith('/') and _path_is_valid(new_root)):
             self.send(src_addr, '/error', path, ray.Err.CREATE_FAILED,
                       "invalid session root !")
@@ -545,10 +576,21 @@ class OscServerThread(ClientCommunicating):
                       "Can't change session_root. Operation pending")
             return False
 
+    @ray_method('/ray/server/set_terminal_command', 's')
+    def rayServerSetTerminalCommand(self, path, args, types, src_addr):
+        if args[0] != self.terminal_command:
+            self.terminal_command = args[0]
+            if not self.terminal_command:
+                self.terminal_command = shlex.join(
+                    which_terminal(title='RAY_TERMINAL_TITLE'))
+            self.send_gui('/ray/gui/server/terminal_command',
+                          self.terminal_command)
+        self.send(src_addr, '/reply', path, 'terminal command set')
+
     @ray_method('/ray/server/list_path', '')
     def rayServerListPath(self, path, args, types, src_addr):
-        exec_list = []
-        tmp_exec_list = []
+        exec_list = list[str]()
+        tmp_exec_list = list[str]()
         n = 0
 
         pathlist = os.getenv('PATH').split(':')
@@ -576,11 +618,11 @@ class OscServerThread(ClientCommunicating):
 
     @ray_method('/ray/server/list_session_templates', '')
     def rayServerListSessionTemplates(self, path, args, types, src_addr):
-        if not os.path.isdir(TemplateRoots.user_sessions):
+        if not TemplateRoots.user_sessions.is_dir():
             self.send(src_addr, '/reply', path)
             return False
 
-        template_list = []
+        template_list = list[str]()
 
         all_files = os.listdir(TemplateRoots.user_sessions)
         for file in all_files:
@@ -613,12 +655,11 @@ class OscServerThread(ClientCommunicating):
 
     @ray_method('/ray/server/remove_client_template', 's')
     def rayServerRemoveClientTemplate(self, path, args, types, src_addr):
-        template_name = args[0]
-
+        template_name: str = args[0]
         templates_root = TemplateRoots.user_clients
-        templates_file = "%s/%s" % (templates_root, 'client_templates.xml')
+        templates_file = templates_root / 'client_templates.xml'
 
-        if not os.path.isfile(templates_file):
+        if not templates_file.is_file():
             self.send(src_addr, '/error', path, ray.Err.NO_SUCH_FILE,
                       "file %s is missing !" % templates_file)
             return False
@@ -628,47 +669,49 @@ class OscServerThread(ClientCommunicating):
                       "file %s in unwriteable !" % templates_file)
             return False
 
-        file = open(templates_file, 'r')
-        xml = QDomDocument()
-        xml.setContent(file.read())
-        file.close()
+        tree = ET.parse(templates_file)
+        root = tree.getroot()
 
-        content = xml.documentElement()
-
-        if content.tagName() != "RAY-CLIENT-TEMPLATES":
+        if root.tag != 'RAY-CLIENT-TEMPLATES':
             self.send(src_addr, '/error', path, ray.Err.BAD_PROJECT,
                       "file %s is not write correctly !" % templates_file)
             return False
 
-        nodes = content.childNodes()
+        xroot = XmlElement(root)
 
-        for i in range(nodes.count()):
-            node = nodes.at(i)
-            ct = node.toElement()
-            tag_name = ct.tagName()
-            if tag_name != 'Client-Template':
+        for c in xroot.iter():
+            if c.el.tag != 'Client-Template':
                 continue
-
-            if template_name == ct.attribute('template-name'):
+            
+            if c.str('template-name') == template_name:
                 break
         else:
             self.send(src_addr, '/error', path, ray.Err.NO_SUCH_FILE,
                       "No template \"%s\" to remove !" % template_name)
             return False
-
-        content.removeChild(nodes.at(i))
-
-        file = open(templates_file, 'w')
-        file.write(xml.toString())
-        file.close()
-
-        template_dir = '%s/%s' % (templates_root, template_name)
-
-        if os.path.isdir(template_dir):
-            subprocess.run(['rm', '-R', template_dir])
+        
+        root.remove(c.el)
+        
+        try:
+            tree.write(templates_file)
+        except BaseException as e:
+            _logger.error(str(e))
+            self.send(src_addr, '/error', path, ray.Err.CREATE_FAILED,
+                      "Impossible to rewrite user client templates xml file")
+            return False
+        
+        templates_dir = templates_root / template_name
+        if templates_dir.is_dir():
+            try:
+                shutil.rmtree(templates_dir)
+            except BaseException as e:
+                _logger.error(str(e))
+                self.send(src_addr, '/error', path, ray.Err.CREATE_FAILED,
+                      "Failed to remove the folder %s" % str(templates_dir))
+                return False
 
         self.send(src_addr, '/reply', path,
-                  "template \"%s\" removed." % template_name)
+                  f'template "{template_name}" removed.')
 
     @ray_method('/ray/server/list_sessions', '')
     def rayServerListSessions(self, path, args, types, src_addr):
@@ -780,11 +823,14 @@ class OscServerThread(ClientCommunicating):
 
     # set options from ray_control
     @ray_method('/ray/server/set_options', None)
-    def rayServerSetOptions(self, path, args, types, src_addr):
+    def rayServerSetOptions(self, path, args, types, src_addr: liblo.Address):
         if not ray.types_are_all_strings(types):
             self._unknown_message(path, types, src_addr)
             return False
 
+        if TYPE_CHECKING:
+            assert isinstance(args, list[str])
+
         for option_str in args:
             option_value = True
             if option_str.startswith('not_'):
@@ -797,12 +843,14 @@ class OscServerThread(ClientCommunicating):
                     if (option == ray.Option.DESKTOPS_MEMORY
                             and not self.options & ray.Option.HAS_WMCTRL):
                         self.send(src_addr, '/minor_error', path,
-                            "wmctrl is not present. Impossible to activate 'desktops_memory' option")
+                            "wmctrl is not present. "
+                            "Impossible to activate 'desktops_memory' option")
                         continue
                     if (option == ray.Option.SNAPSHOTS
                             and not self.options & ray.Option.HAS_GIT):
                         self.send(src_addr, '/minor_error', path,
-                            "git is not present. Impossible to activate 'snapshots' option")
+                            "git is not present. "
+                            "Impossible to activate 'snapshots' option")
                         continue
 
                 if not option_value:
@@ -847,21 +895,22 @@ class OscServerThread(ClientCommunicating):
     @ray_method('/ray/server/exotic_action', 's')
     def rayServerExoticAction(self, path, args, types, src_addr):
         action = args[0]
-        autostart_dir = "%s/.config/autostart" % os.getenv('HOME')
+        autostart_dir = Path.home() / '.config' / 'autostart'
         desk_file = "ray-jack_checker.desktop"
+        dest_full_path = autostart_dir / desk_file
 
         if action == 'set_jack_checker_autostart':
-            if not os.path.exists(autostart_dir):
-                os.makedirs(autostart_dir)
-
-            src_full_file = "%s/data/share/applications/%s" % (get_code_root(),
-                                                               desk_file)
-            dest_full_path = "%s/%s" % (autostart_dir, desk_file)
+            if not autostart_dir.exists():
+                autostart_dir.mkdir(parents=True)
 
+            src_full_file = (
+                get_code_root()
+                / 'data' / 'share' / 'applications' / desk_file)
+            
             shutil.copyfile(src_full_file, dest_full_path)
 
         elif action == 'unset_jack_checker_autostart':
-            os.remove("%s/%s" % (autostart_dir, desk_file))
+            dest_full_path.unlink(missing_ok=True)
 
     @ray_method('/ray/server/patchbay/save_group_position',
                 ray.GroupPosition.sisi())
@@ -873,7 +922,7 @@ class OscServerThread(ClientCommunicating):
                           *args)
 
     @ray_method('/ray/server/patchbay/save_portgroup', None)
-    def rayServerPatchbaySavePortGroup(self, path, args, types, src_addr):
+    def rayServerPatchbaySavePortGroup(self, path, args, types: str, src_addr):
         # args must be group_name, port_type, port_mode, above_metadatas, *port_names
         # where port_names are all strings
         # so types must start with 'siiis' and may continue with strings only
@@ -1053,7 +1102,7 @@ class OscServerThread(ClientCommunicating):
                 "Absolute paths are not permitted. Clients must be in $PATH")
             return False
 
-    @ray_method('/ray/session/add_client_template', 'is')
+    @ray_method('/ray/session/add_client_template', 'isss')
     def rayServerAddClientTemplate(self, path, args, types, src_addr):
         pass
 
@@ -1219,6 +1268,18 @@ class OscServerThread(ClientCommunicating):
             self._unknown_message(path, types, src_addr)
             return False
 
+    @ray_method('/ray/client/change_advanced_properties', 'ssisi')
+    def rayClientChangeAdvancedProperties(self, path, args, types, src_addr):
+        pass
+    
+    @ray_method('/ray/client/full_rename', 'ss')
+    def rayClientFullRename(self, path, args, types, src_addr):
+        pass
+    
+    @ray_method('/ray/client/change_id', 'ss')
+    def rayClientChangeId(self, path, args, types, src_addr):
+        pass
+
     @ray_method('/ray/client/set_description', 'ss')
     def rayClientSetDescription(self, path, args, types, src_addr):
         pass
@@ -1283,17 +1344,19 @@ class OscServerThread(ClientCommunicating):
     def rayDuplicateState(self, path, args, types, src_addr):
         pass
 
-    @ray_method('/ray/favorites/add', 'ssi')
+    @ray_method('/ray/favorites/add', 'ssis')
     def rayFavoriteAdd(self, path, args, types, src_addr):
-        name, icon, int_factory = args
+        name, icon, int_factory, display_name = args
 
         for favorite in RS.favorites:
             if (favorite.name == name
                     and bool(int_factory) == favorite.factory):
                 favorite.icon = icon
+                favorite.display_name = display_name
                 break
         else:
-            RS.favorites.append(ray.Favorite(name, icon, bool(int_factory)))
+            RS.favorites.append(
+                ray.Favorite(name, icon, bool(int_factory), display_name))
 
         self.send_gui('/ray/gui/favorites/added', *args)
 
@@ -1380,12 +1443,15 @@ class OscServerThread(ClientCommunicating):
         self.send(gui_addr, "/ray/gui/session/name",
                   self.session.name, self.session.path)
         self.send(gui_addr, '/ray/gui/session/notes', self.session.notes)
+        self.send(gui_addr, '/ray/gui/server/terminal_command',
+                  self.terminal_command)
 
         self.session.canvas_saver.send_all_group_positions(gui_addr)
 
         for favorite in RS.favorites:
             self.send(gui_addr, "/ray/gui/favorites/added",
-                      favorite.name, favorite.icon, int(favorite.factory))
+                      favorite.name, favorite.icon, int(favorite.factory),
+                      favorite.display_name)
 
         for client in self.session.clients:
             self.send(gui_addr,
@@ -1470,14 +1536,14 @@ class OscServerThread(ClientCommunicating):
 
         return 0
 
-    def get_local_gui_pid_list(self)->str:
+    def get_local_gui_pid_list(self) -> str:
         pid_list = []
         for gui_addr in self.gui_list:
             if ray.are_on_same_machine(gui_addr.url, self.url):
                 pid_list.append(str(gui_addr.gui_pid))
         return ':'.join(pid_list)
 
-    def is_gui_address(self, addr)->bool:
+    def is_gui_address(self, addr: liblo.Address) -> bool:
         for gui_addr in self.gui_list:
             if ray.are_same_osc_port(gui_addr.url, addr.url):
                 return True
