@@ -1786,6 +1786,7 @@ for better organization.""")
         sess_name = ""
 
         if is_ray_file:
+            print('marrouubma')
             try:
                 tree = ET.parse(session_ray_file)
             except BaseException as e:
@@ -1829,7 +1830,7 @@ for better organization.""")
 
                 elif child.tag == 'Windows':
                     if self.has_server_option(ray.Option.DESKTOPS_MEMORY):
-                        self.desktops_memory.read_xml(c)
+                        self.desktops_memory.read_xml(XmlElement(child))
 
         else:
             # prevent to load a locked NSM session
