@@ -1,13 +1,23 @@
 
 import os
 import socket
+import logging
+from pathlib import Path
+from typing import TYPE_CHECKING, Callable, Optional
 from PyQt5.QtCore import QProcess, QObject, QDateTime
-from PyQt5.QtXml import QDomDocument
+from PyQt5.QtXml import QDomDocument, QDomElement
+import xml.etree.ElementTree as ET
 
 import ray
 from daemon_tools import Terminal
+from xml_tools import XmlElement
 
-def git_stringer(string:str)->str:
+if TYPE_CHECKING:
+    from session import Session
+    
+_logger = logging.getLogger(__name__)
+
+def git_stringer(string:str) -> str:
     for char in (' ', '*', '?', '[', ']', '(', ')'):
         string = string.replace(char, "\\" + char)
 
@@ -24,7 +34,7 @@ def full_ref_for_gui(ref, name, rw_ref, rw_name='', ss_name=''):
 
 
 class Snapshoter(QObject):
-    def __init__(self, session):
+    def __init__(self, session: 'Session'):
         QObject.__init__(self)
         self.session = session
         self._git_exec = 'git'
@@ -83,10 +93,10 @@ class Snapshoter(QObject):
         standard_output = self._git_process.readAllStandardOutput().data()
         Terminal.snapshoter_message(standard_output, self._git_command)
 
-    def _run_git_process(self, *all_args):
+    def _run_git_process(self, *all_args) -> int:
         return self._run_git_process_at(self.session.path, *all_args)
 
-    def _run_git_process_at(self, spath, *all_args):
+    def _run_git_process_at(self, spath, *all_args) -> int:
         self._git_command = ''
         for arg in all_args:
             self._git_command += ' %s' % arg
@@ -109,10 +119,10 @@ class Snapshoter(QObject):
 
         return not bool(err)
 
-    def _get_git_command_list(self, *args):
+    def _get_git_command_list(self, *args) -> list[str]:
         return self._get_git_command_list_at(self.session.path, *args)
 
-    def _get_git_command_list_at(self, spath, *args):
+    def _get_git_command_list_at(self, spath: str, *args) -> list[str]:
         first_args = ['--work-tree', spath, '--git-dir',
                       "%s/%s" % (spath, self._gitdir)]
 
@@ -122,7 +132,7 @@ class Snapshoter(QObject):
         return "%s/%s/%s" % (
                         self.session.path, self._gitdir, self._history_path)
 
-    def _get_history_xml_document_element(self):
+    def _get_history_xml_document_element(self) -> Optional[QDomElement]:
         if not self._is_init():
             return None
 
@@ -154,64 +164,54 @@ class Snapshoter(QObject):
 
         return tagdate
 
-    def _write_history_file(self, date_str, snapshot_name='', rewind_snapshot=''):
+    def _write_history_file(self, date_str: str, snapshot_name='', rewind_snapshot='') -> int:
         if not self.session.path:
             return ray.Err.NO_SESSION_OPEN
 
         file_path = self._get_history_full_path()
 
-        xml = QDomDocument()
-
         try:
-            history_file = open(file_path, 'r')
-            xml.setContent(history_file.read())
-            history_file.close()
-        except:
-            pass
-
-        if xml.firstChild().isNull():
-            SNS_xml = xml.createElement('SNAPSHOTS')
-            xml.appendChild(SNS_xml)
-        else:
-            SNS_xml = xml.firstChild()
-
-        snapshot_el = xml.createElement('Snapshot')
-        snapshot_el.setAttribute('ref', date_str)
-        snapshot_el.setAttribute('name', snapshot_name)
-        snapshot_el.setAttribute('rewind_snapshot', rewind_snapshot)
-        snapshot_el.setAttribute('session_name', self.session.name)
-        snapshot_el.setAttribute('VERSION', ray.VERSION)
+            tree = ET.parse(file_path)
+            root = tree.getroot()
+        except BaseException as e:
+            _logger.info(str(e))
+            root = ET.Element('SNAPSHOTS')
+            tree = ET.ElementTree(root)
 
+        root.tag = 'SNAPSHOTS'
+        snapshot_el = ET.SubElement(root, 'Snapshot')
+        s = XmlElement(snapshot_el)
+        s.set_str('ref', date_str)
+        s.set_str('name', snapshot_name)
+        s.set_str('rewind_snapshot', rewind_snapshot)
+        s.set_str('session_name', self.session.name)
+        s.set_str('VERSION', ray.VERSION)
+        
         for client in self.session.clients + self.session.trashed_clients:
-            client_el = xml.createElement('client')
-            client.write_xml_properties(client_el)
-            client_el.setAttribute('client_id', client.client_id)
-
+            client_el = ET.SubElement(snapshot_el, 'client')
+            c = XmlElement(client_el)
+            client.write_xml_properties(c)
+            c.set_str('client_id', client.client_id)
+            
             for client_file_path in client.get_project_files():
-                base_path = client_file_path.replace(
-                    "%s/" % self.session.path, '', 1)
-                file_xml = xml.createElement('file')
-                file_xml.setAttribute('path', base_path)
-                client_el.appendChild(file_xml)
-
-            snapshot_el.appendChild(client_el)
-
-        SNS_xml.appendChild(snapshot_el)
-
+                base_path = str(
+                    Path(client_file_path).relative_to(self.session.path))
+                file_xml = ET.SubElement(client_el, 'file')
+                fxl = XmlElement(file_xml)
+                fxl.set_str('path', base_path)
+                
         try:
-            history_file = open(file_path, 'w')
-            history_file.write(xml.toString())
-            history_file.close()
-        except:
+            tree.write(file_path)
+            return ray.Err.OK
+        except BaseException as e:
+            _logger.error(str(e))
             return ray.Err.CREATE_FAILED
 
-        return ray.Err.OK
-
-    def _get_exclude_file_full_path(self)->str:
+    def _get_exclude_file_full_path(self) -> str:
         return "%s/%s/%s" % (
                         self.session.path, self._gitdir, self._exclude_path)
 
-    def _write_exclude_file(self)->int:
+    def _write_exclude_file(self) -> int:
         file_path = self._get_exclude_file_full_path()
 
         try:
@@ -323,14 +323,14 @@ class Snapshoter(QObject):
 
         return ray.Err.OK
 
-    def _is_init(self)->bool:
+    def _is_init(self) -> bool:
         if not self.session.path:
             return False
 
         return os.path.isfile("%s/%s/%s" % (
                 self.session.path, self._gitdir, self._exclude_path))
 
-    def _can_save(self):
+    def _can_save(self) -> bool:
         if not self.session.path:
             return False
 
@@ -377,8 +377,6 @@ class Snapshoter(QObject):
             if not self._run_git_process('commit', '-m', 'ray'):
                 return
 
-
-
         if (self._n_file_changed
                 or self._next_snapshot_name or self._rw_snapshot):
             ref = self._get_tag_date()
@@ -403,15 +401,15 @@ class Snapshoter(QObject):
         if self._next_function:
             self._next_function()
 
-    def list(self, client_id=""):
+    def list(self, client_id="") -> list[str]:
         SNS_xml = self._get_history_xml_document_element()
         if not SNS_xml:
             return []
 
         nodes = SNS_xml.childNodes()
 
-        all_tags = []
-        all_snaps = []
+        all_tags = list[str]()
+        all_snaps = list[tuple[str, str]]()
         prv_session_name = self.session.name
 
         for i in range(nodes.count()):
@@ -470,7 +468,7 @@ class Snapshoter(QObject):
         all_tags.reverse()
         return all_tags
 
-    def has_changes(self):
+    def has_changes(self) -> bool:
         if not self.session.path:
             return False
 
@@ -522,7 +520,7 @@ class Snapshoter(QObject):
 
         # self.adder_process.finished is connected to self._save_step_1
 
-    def load(self, spath, snapshot, error_function):
+    def load(self, spath: str, snapshot: str, error_function: Callable):
         self._error_function = error_function
 
         snapshot_ref = snapshot.partition('\n')[0].partition(':')[0]
@@ -534,7 +532,8 @@ class Snapshoter(QObject):
             return False
         return True
 
-    def load_client_exclusive(self, client_id, snapshot, error_function):
+    def load_client_exclusive(self, client_id: str, snapshot: str,
+                              error_function: Callable):
         self._error_function = error_function
 
         SNS_xml = self._get_history_xml_document_element()
@@ -545,7 +544,7 @@ class Snapshoter(QObject):
 
         nodes = SNS_xml.childNodes()
 
-        client_path_list = []
+        client_path_list = list[str]()
 
         for i in range(nodes.count()):
             node = nodes.at(i)
