@@ -2,51 +2,51 @@ import os
 import sys
 import time
 import subprocess
+from typing import TYPE_CHECKING
 
 from PyQt5.QtWidgets import (
-    QDialog, QDialogButtonBox, QTreeWidget, QTreeWidgetItem,
-    QCompleter, QMessageBox, QFileDialog, QApplication, QListWidgetItem)
-from PyQt5.QtGui import QIcon, QPixmap, QGuiApplication
+    QDialog, QDialogButtonBox, QCompleter, QMessageBox,
+    QFileDialog, QApplication, QListWidgetItem)
+from PyQt5.QtGui import QIcon, QPixmap, QGuiApplication, QKeyEvent
 from PyQt5.QtCore import Qt, QTimer
 
-import client_properties_dialog
 import ray
+import client_properties_dialog
 from gui_server_thread import GuiServerThread
 from gui_tools import (ErrDaemon, _translate, get_app_icon,
                        CommandLineArgs, RS, is_dark_theme)
-
-from patchcanvas import patchcanvas
-
+if TYPE_CHECKING:
+    from main_window import MainWindow
+    from gui_client import Client
 
 import ui.new_session
 import ui.save_template_session
-import ui.nsm_open_info
+import ui.client_trash
 import ui.abort_session
-import ui.about_raysession
-import ui.donations
-import ui.jack_config_info
-import ui.new_executable
-import ui.error_dialog
-import ui.quit_app
-import ui.script_info
-import ui.script_user_action
+import ui.abort_copy
 import ui.session_notes
-import ui.session_scripts_info
+import ui.nsm_open_info
+import ui.quit_app
+import ui.about_raysession
+import ui.new_executable
 import ui.stop_client
 import ui.stop_client_no_save
-import ui.abort_copy
-import ui.client_trash
-import ui.daemon_url
-import ui.snapshot_progress
-import ui.waiting_close_user
 import ui.client_rename
-import ui.canvas_port_info
-import ui.startup_dialog
+import ui.snapshot_progress
+import ui.script_info
+import ui.script_user_action
+import ui.session_scripts_info
+import ui.jack_config_info
+import ui.daemon_url
+import ui.waiting_close_user
+import ui.donations
 import ui.systray_close
-import ui.systray_management
+import ui.startup_dialog
+import ui.error_dialog
+
 
 class ChildDialog(QDialog):
-    def __init__(self, parent):
+    def __init__(self, parent: 'MainWindow'):
         QDialog.__init__(self, parent)
         self.session = parent.session
         self.signaler = self.session.signaler
@@ -126,13 +126,20 @@ class ChildDialog(QDialog):
         RS.settings.setValue('default_session_root', root_folder)
         self.to_daemon('/ray/server/change_root', root_folder)
 
+    def parent(self) -> 'MainWindow':
+        super().parent()
+
     def leaveEvent(self, event):
-        if self.isActiveWindow():
-            self.parent().mouse_is_inside = False
+        parent = self.parent()
+        
+        if parent is not None and self.isActiveWindow():
+            parent.mouse_is_inside = False
         QDialog.leaveEvent(self, event)
 
     def enterEvent(self, event):
-        self.parent().mouse_is_inside = True
+        parent = self.parent()
+        if parent is not None:
+            parent.mouse_is_inside = True
         QDialog.enterEvent(self, event)
 
 
@@ -237,7 +244,7 @@ class NewSessionDialog(ChildDialog):
                                     len(ray.FACTORY_SESSION_TEMPLATES) + 1)
 
     def _set_last_template_selected(self):
-        last_used_template = RS.settings.value('last_used_template', type=str)
+        last_used_template: str = RS.settings.value('last_used_template', type=str)
 
         if last_used_template.startswith('///'):
             last_factory_template = last_used_template.replace('///', '', 1)
@@ -303,7 +310,7 @@ class NewSessionDialog(ChildDialog):
 
         self._set_last_template_selected()
 
-    def _text_changed(self, text):
+    def _text_changed(self, text: str):
         self._text_is_valid = bool(text
                                    and not text.endswith('/')
                                    and text not in self.session_list)
@@ -320,7 +327,6 @@ class NewSessionDialog(ChildDialog):
             self._text_was_empty = True
 
     def _set_completer_for_empty_text(self):
-        #print('set currow', self._completer.setCurrentIndex(0))
         del self._completer
         self._completer = QCompleter([f + '/' for f in self.sub_folders])
         self._completer.setCompletionMode(QCompleter.UnfilteredPopupCompletion)
@@ -378,7 +384,7 @@ class AbstractSaveTemplateDialog(ChildDialog):
         self.ui.pushButtonAccept.clicked.connect(self._verify_and_accept)
         self.ui.pushButtonAccept.setEnabled(False)
 
-    def _text_edited(self, text):
+    def _text_edited(self, text: str):
         if '/' in text:
             self.ui.lineEdit.setText(text.replace('/', '⁄'))
         if self.ui.lineEdit.text() in self.template_list:
@@ -441,7 +447,7 @@ class SaveTemplateSessionDialog(AbstractSaveTemplateDialog):
 
 
 class SaveTemplateClientDialog(AbstractSaveTemplateDialog):
-    def __init__(self, parent, client):
+    def __init__(self, parent, client: 'Client'):
         AbstractSaveTemplateDialog.__init__(self, parent)
         self.ui.labelSessionTitle.setVisible(False)
         self.ui.toolButtonClientIcon.setIcon(
@@ -478,7 +484,7 @@ class SaveTemplateClientDialog(AbstractSaveTemplateDialog):
 
 
 class ClientTrashDialog(ChildDialog):
-    def __init__(self, parent, client_data):
+    def __init__(self, parent, client_data: ray.ClientData):
         ChildDialog.__init__(self, parent)
         self.ui = ui.client_trash.Ui_Dialog()
         self.ui.setupUi(self)
@@ -646,7 +652,8 @@ class SessionNotesDialog(ChildDialog):
                 QMessageBox.Critical,
                 _translate('session_notes', 'Too long notes'),
                 _translate('session_notes',
-                           "<p>Because notes are spread to the OSC server,<br>they can't be longer than 65000 characters.<br>Sorry !</p>"),
+                           "<p>Because notes are spread to the OSC server,<br>"
+                           "they can't be longer than 65000 characters.<br>Sorry !</p>"),
                 QMessageBox.Cancel,
                 self)
             self._message_box.exec()
@@ -672,6 +679,7 @@ class SessionNotesDialog(ChildDialog):
             self.to_daemon('/ray/session/hide_notes')
         ChildDialog.closeEvent(self, event)
 
+
 class OpenNsmSessionInfoDialog(ChildDialog):
     def __init__(self, parent):
         ChildDialog.__init__(self, parent)
@@ -833,7 +841,7 @@ class NewExecutableDialog(ChildDialog):
     def _prefix_mode_changed(self, index: int):
         self.ui.lineEditPrefix.setEnabled(bool(index == 0))
 
-    def _add_executable_to_completer(self, executable_list):
+    def _add_executable_to_completer(self, executable_list: list):
         self.exec_list += executable_list
         self.exec_list.sort()
 
@@ -841,12 +849,10 @@ class NewExecutableDialog(ChildDialog):
         self._completer = QCompleter(self.exec_list)
         self.ui.lineEdit.setCompleter(self._completer)
 
-    def _is_allowed(self):
+    def _is_allowed(self) -> bool:
         nsm = self.ui.checkBoxNsm.isChecked()
         text = self.ui.lineEdit.text()
-        allow = bool(bool(text) and (not nsm
-                                     or text in self.exec_list))
-        return allow
+        return bool(bool(text) and (not nsm or text in self.exec_list))
 
     def _check_allow(self):
         allow = self._is_allowed()
@@ -884,9 +890,9 @@ class StopClientDialog(ChildDialog):
                 minutes = int((time.time() - self.client.last_save) / 60)
                 text = _translate(
                     'client_stop',
-                    "<strong>%s</strong> seems to has not been saved for %i minute(s).<br />Do you really want to stop it ?") \
+                    "<strong>%s</strong> seems to has not been saved for %i minute(s).<br />"
+                    "Do you really want to stop it ?") \
                         % (self.client.prettier_name(), minutes)
-
             self.ui.label.setText(text)
 
             self.client.status_changed.connect(self._server_updates_client_status)
@@ -939,7 +945,7 @@ class StopClientNoSaveDialog(ChildDialog):
 
 
 class ClientRenameDialog(ChildDialog):
-    def __init__(self, parent, client):
+    def __init__(self, parent, client: 'Client'):
         ChildDialog.__init__(self, parent)
         self.ui = ui.client_rename.Ui_Dialog()
         self.ui.setupUi(self)
@@ -950,10 +956,74 @@ class ClientRenameDialog(ChildDialog):
         self.ui.lineEdit.setText(client.prettier_name())
         self.ui.lineEdit.selectAll()
         self.ui.lineEdit.setFocus()
+        self.ui.lineEdit.textEdited.connect(self._text_edited)
+        self.ui.checkBoxIdRename.stateChanged.connect(
+            self._id_rename_state_changed)
 
-    def get_new_label(self)->str:
+        if client.protocol not in (ray.Protocol.NSM, ray.Protocol.RAY_HACK):
+            self.ui.checkBoxIdRename.setVisible(False)
+
+        self._change_box_text_with_status(client.status)
+        client.status_changed.connect(self._client_status_changed)
+
+    def _change_box_text_with_status(self, status: int):
+        can_switch = ':switch:' in self.client.capabilities
+
+        if status in (
+                ray.ClientStatus.STOPPED, ray.ClientStatus.PRECOPY,
+                ray.ClientStatus.QUIT, ray.ClientStatus.LOSE):
+            text = ''
+        elif status == ray.ClientStatus.READY and can_switch:
+            text = _translate(
+                'id_renaming', 'The client project will be reload')
+        else:
+            text = _translate(
+                'id_renaming', 'The client will be restarted')
+        
+        full_text = _translate('id_renaming', 'Rename Identifier')
+        if text:
+            full_text += f'\n({text})'
+        
+        self.ui.checkBoxIdRename.setText(full_text)    
+
+    def _client_status_changed(self, status: int):
+        if status == ray.ClientStatus.REMOVED:
+            self.reject()
+            
+        self._change_box_text_with_status(status)
+    
+    def _id_rename_state_changed(self, state: int):
+        if state:
+            self._text_edited(self.ui.lineEdit.text())
+    
+    def _text_edited(self, text: str):
+        if not self.is_identifiant_renamed():
+            return
+        
+        out_text = ''.join([c for c in text if c.isalnum() or c == ' '])
+
+        if out_text != text:
+            self.ui.lineEdit.setText(out_text)
+        
+        out_id = out_text.replace(' ', '_')
+        session = self.client.session
+        ok = True
+        
+        for cl in session.client_list + session.trashed_clients:
+            cl: Client
+            if cl.client_id == out_id:
+                self.ui.buttonBox.button(
+                    QDialogButtonBox.Ok).setEnabled(False)
+                return
+        
+        self.ui.buttonBox.button(QDialogButtonBox.Ok).setEnabled(True)
+    
+    def is_identifiant_renamed(self) -> bool:
+        return self.ui.checkBoxIdRename.isChecked()
+
+    def get_new_label(self) -> str:
         return self.ui.lineEdit.text()
-
+    
 
 class SnapShotProgressDialog(ChildDialog):
     def __init__(self, parent):
@@ -1156,28 +1226,6 @@ class WaitingCloseUserDialog(ChildDialog):
         RS.set_hidden(RS.HD_WaitCloseUser, bool(state))
 
 
-class CanvasPortInfoDialog(ChildDialog):
-    def __init__(self, parent):
-        ChildDialog.__init__(self, parent)
-        self.ui = ui.canvas_port_info.Ui_Dialog()
-        self.ui.setupUi(self)
-
-    def set_infos(self, port_full_name: str, port_uuid: int,
-                  port_type: str, port_flags: str,
-                  pretty_name: str, port_order: int,
-                  portgroup_name: str):
-        self.ui.lineEditFullPortName.setText(port_full_name)
-        self.ui.lineEditUuid.setText(str(port_uuid))
-        self.ui.labelPortType.setText(port_type)
-        self.ui.labelPortFlags.setText(port_flags)
-        self.ui.labelPrettyName.setText(pretty_name)
-        self.ui.labelPortOrder.setText(port_order)
-        self.ui.labelPortGroup.setText(portgroup_name)
-
-        if not (pretty_name or port_order or portgroup_name):
-            self.ui.groupBoxMetadatas.setVisible(False)
-
-
 class DonationsDialog(ChildDialog):
     def __init__(self, parent, display_no_again):
         ChildDialog.__init__(self, parent)
@@ -1201,41 +1249,6 @@ class SystrayCloseDialog(ChildDialog):
         return self.ui.checkBox.isChecked()
 
 
-class SystrayManagement(ChildDialog):
-    def __init__(self, parent):
-        ChildDialog.__init__(self, parent)
-        self.ui = ui.systray_management.Ui_Dialog()
-        self.ui.setupUi(self)
-
-        self.ui.groupBoxSystray.toggled.connect(
-            self.ui.checkBoxOnlySessionRunning.setEnabled)
-
-    def set_systray_mode(self, systray_mode: int):
-        self.ui.groupBoxSystray.setChecked(
-            systray_mode != ray.Systray.OFF)
-        self.ui.checkBoxOnlySessionRunning.setChecked(
-            systray_mode == ray.Systray.SESSION_ONLY)
-
-    def set_wild_shutdown(self, wild_shutdown: bool):
-        self.ui.checkBoxShutdown.setChecked(wild_shutdown)
-
-    def set_reversed_state(self, reversed: bool):
-        self.ui.checkBoxReversedMenu.setChecked(reversed)
-
-    def get_systray_mode(self)->int:
-        if self.ui.groupBoxSystray.isChecked():
-            if self.ui.checkBoxOnlySessionRunning.isChecked():
-                return ray.Systray.SESSION_ONLY
-            return ray.Systray.ALWAYS
-        return ray.Systray.OFF
-
-    def wild_shutdown(self)->bool:
-        return self.ui.checkBoxShutdown.isChecked()
-
-    def menu_should_be_reversed(self)->bool:
-        return self.ui.checkBoxReversedMenu.isChecked()
-
-
 class StartupDialog(ChildDialog):
     ACTION_NO = 0
     ACTION_NEW = 1
@@ -1309,7 +1322,7 @@ class StartupDialog(ChildDialog):
     def get_clicked_action(self)->int:
         return self._clicked_action
 
-    def keyPressEvent(self, event):
+    def keyPressEvent(self, event: QKeyEvent):
         if event.key() == Qt.Key_Left:
             self.ui.pushButtonNewSession.setFocus(Qt.OtherFocusReason)
         elif event.key() == Qt.Key_Right:
