@@ -2,6 +2,8 @@
 import os
 import socket
 import sys
+import time
+from typing import TYPE_CHECKING
 from PyQt5.QtCore import QObject, QProcess, QTimer
 from PyQt5.QtWidgets import QApplication
 from liblo import Address
@@ -10,8 +12,12 @@ import ray
 from gui_server_thread import GuiServerThread
 from gui_tools import CommandLineArgs, ErrDaemon, _translate
 
+if TYPE_CHECKING:
+    from .gui_session import SignaledSession
+
+
 class DaemonManager(QObject):
-    def __init__(self, session):
+    def __init__(self, session: 'SignaledSession'):
         QObject.__init__(self)
         self.session = session
         self.signaler = self.session.signaler
@@ -86,8 +92,8 @@ class DaemonManager(QObject):
             QApplication.quit()
 
     def _receive_announce(
-            self, src_addr, version, server_status,
-            options, session_root, is_net_free):
+            self, src_addr: Address, version: str, server_status: int,
+            options: int, session_root: str, is_net_free: int):
         self._announce_timer.stop()
 
         if version.split('.')[:2] != ray.VERSION.split('.')[:2]:
@@ -160,7 +166,7 @@ class DaemonManager(QObject):
     def set_external(self):
         self.launched_before = True
 
-    def set_osc_address(self, address):
+    def set_osc_address(self, address: Address):
         self.address = address
         self.launched_before = True
         self._port = self.address.port
