@@ -1,12 +1,18 @@
 import argparse
 import os
+from pathlib import Path
 import sys
-from PyQt5.QtCore import QSettings, QSize, QFile
+from typing import TYPE_CHECKING
+from PyQt5.QtCore import QSettings, QSize, QFile, QObject, pyqtSignal
 from PyQt5.QtWidgets import QApplication, QWidget
 from PyQt5.QtGui import QIcon, QPixmap, QPalette
+from liblo import Address
 
 import ray
 
+if TYPE_CHECKING:
+    from gui_signaler import Signaler
+
 _translate = QApplication.translate
 
 _RAY_ICONS_CACHE_LIGHT = {}
@@ -29,7 +35,9 @@ class RS:
     HD_SystrayClose = 0x040
     HD_StartupRecentSessions = 0x080
     HD_ArdourConversion = 0x100
-
+    
+    _signaler: 'Signaler' = None
+    
     @classmethod
     def set_settings(cls, settings):
         del cls.settings
@@ -50,10 +58,19 @@ class RS:
             hidden_dialogs &= ~hiddeable_dialog
 
         cls.settings.setValue('hidden_dialogs', hidden_dialogs)
+        
+        if cls._signaler is not None:
+            cls._signaler.hiddens_changed.emit(hidden_dialogs)
 
     @classmethod
     def reset_hiddens(cls):
         cls.settings.setValue('hidden_dialogs', 0)
+        if cls._signaler is not None:
+            cls._signaler.hiddens_changed.emit(0)
+        
+    @classmethod
+    def set_signaler(cls, signaler: 'Signaler'):
+        cls._signaler = signaler
 
 
 class ErrDaemon:
@@ -77,6 +94,7 @@ class RayAbstractIcon(QIcon):
                 ':scalable/%s/disabled/%s' %
                 (breeze, icon_name)), QIcon.Disabled, QIcon.Off)
 
+
 def RayIcon(icon_name: str, dark=False) -> RayAbstractIcon:
     if dark and icon_name in _RAY_ICONS_CACHE_DARK.keys():
         return _RAY_ICONS_CACHE_DARK[icon_name]
@@ -93,8 +111,10 @@ def RayIcon(icon_name: str, dark=False) -> RayAbstractIcon:
 
 
 class CommandLineArgs(argparse.Namespace):
-    daemon_url = None
+    daemon_url: Address = None
+    daemon_port: Address = None
     out_daemon = False
+    session_root: str = ''
     config_dir = ''
     debug = False
     debug_only = False
@@ -255,8 +275,8 @@ def dirname(*args) -> str:
 def basename(*args) -> str:
     return os.path.basename(*args)
 
-def get_code_root() -> str:
-    return dirname(dirname(dirname(os.path.realpath(__file__))))
+def get_code_root() -> Path:
+    return Path(__file__).parent.parent.parent
 
 def server_status_string(server_status: int) -> str:
     server_status_strings = {
@@ -273,10 +293,10 @@ def server_status_string(server_status: int) -> str:
         ray.ServerStatus.CLOSE   : _translate('server status', "close"),
         ray.ServerStatus.SNAPSHOT: _translate('server_status', "snapshot"),
         ray.ServerStatus.REWIND  : _translate('server_status', "rewind"),
-        ray.ServerStatus.WAIT_USER : _translate('server_status', "waiting"),
-        ray.ServerStatus.OUT_SAVE  : _translate('server_status', "save"),
+        ray.ServerStatus.WAIT_USER   : _translate('server_status', "waiting"),
+        ray.ServerStatus.OUT_SAVE    : _translate('server_status', "save"),
         ray.ServerStatus.OUT_SNAPSHOT: _translate('server_status', "snapshot"),
-        ray.ServerStatus.SCRIPT : _translate('server_status', "script")}
+        ray.ServerStatus.SCRIPT  : _translate('server_status', "script")}
 
     if not 0 <= server_status < len(server_status_strings):
         return _translate('server status', "invalid")
@@ -298,7 +318,8 @@ def client_status_string(client_status: int) -> str:
         ray.ClientStatus.ERROR  : _translate('client status', "error"),
         ray.ClientStatus.REMOVED: _translate('client status', "removed"),
         ray.ClientStatus.UNDEF  : _translate('client_status', ""),
-        ray.ClientStatus.SCRIPT : _translate('client_status', 'script')}
+        ray.ClientStatus.SCRIPT : _translate('client_status', 'script'),
+        ray.ClientStatus.LOSE   : _translate('client_status', "lose")}
 
     if not 0 <= client_status < len(client_status_strings):
         return _translate('client_status', 'invalid')
